#!/usr/bin/env python3

import glob
import re

#print("\n\n\n-------------------------------\n\n\n")

type_uses = {}

# Find all Actors
for filename in glob.glob("./*/src/*.rs"):
    # ignore old code
    if filename.startswith("./old/"):
        continue
    #print(filename)
    with open(filename) as f:
        lines = f.readlines()
        
        next_letters = re.compile("^[A-Za-z0-9_]+")
        in_brackets = re.compile("\(.+\)")
        in_type = False
        type_name = None
        for line in lines:
            line = line.strip()
            if in_type:
                if line.startswith("//") or line.startswith("#"):
                    continue
                if line == "}":
                    in_type = False
                    continue
                else:
                    (field_name, field_type) = line.split(": ", 1)
                    type_uses[type_name].add(field_type[:-1])
                    continue
            
            name_plus = None
            if line.startswith("pub struct"):
                (_, a) = line.split("pub struct ", 1)
                name_plus = a
                
            if line.startswith("pub(crate) struct "):
                (_, name_plus) = line.split("pub(crate) struct ", 1)
                name_plus = a
            
            if name_plus:
                find = next_letters.match(name_plus)
                name = find.group()
                end = name_plus[find.end():]
                #print(" T: ", name, " <-> ", end)
                type_name = name
                
                if end.endswith(");"):
                    inner = in_brackets.match(end).group()
                    inner = str(inner)[1:-1]
                    # remove keywords
                    inner = inner.replace("pub", "").replace("(crate)", "").replace("dyn", "")
                    inner = inner.split(", ")
                    inner = set( map(str.strip, inner) )
                    #print("   -> inner:", inner)
                    type_uses[name] = inner
                else:
                    in_type = True
                    type_uses[name] = set()

#print("found", type_uses)

# Parse Generics
all_types = set()
for (name, types) in type_uses.items():
    parts = set()
    for t in types:
        if "<" in t:
            t = t.replace("<", " ").replace(">", " ").strip().split()
            t = map(lambda s: s.replace(",", "").strip(), t)
            parts |= set(t)
        else:
            parts.add(t)
    type_uses[name] = parts
    all_types |= parts

print("digraph ActorAccepts {")

#print("    {rank=same;", " ".join(actor_accepts.keys()),"}")
for name in type_uses.keys():
    print("    ", name, "[fontcolor=red,style=filled]")

for std in ["Arc", "Weak", "Mutex", "u8", "u64", "u128", "usize", "Vec", "String", "str", "PathBuf", "Path", "Box", "AtomicUsize", "HashMap", "HashSet", "Option", "StdFile", "Rc", "RefCell", "Cell", "bool"]:
    if std in all_types:
        print("    ", std, "[fontcolor=blue]")

for (name, types) in type_uses.items():
    types = ", ".join( map(lambda s: '"{}"'.format(s), types) )
    print("    {} -> {{ {} }}".format(name, types))

print("}")

